var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import fs from 'fs-extra';
import * as path from 'path';
import { AsyncSeriesWaterfallHook } from 'tapable';
import { Log } from 'codegem-tools';
// machine 运行完成，生成代码以后的钩子
const generatedHook = new AsyncSeriesWaterfallHook(['name']);
export default class Generator {
    constructor(option, ctx) {
        this.storage = {};
        this.factory = [];
        this.option = option;
        this.ctx = ctx;
        this.log = new Log(ctx);
    }
    init() {
        var _a;
        this.log.info('init');
        let factories = this.option.factory;
        if (this.ctx.name) {
            factories = this.option.factory.filter((it) => it.name === this.ctx.name);
        }
        if (factories.length > 0) {
            factories.forEach((factory, index) => {
                const factoryId = Symbol(index);
                this.factory.push(Object.assign(Object.assign({}, factory), { id: factoryId }));
                this.createStorage(factoryId, []);
            });
            (_a = this.option.plugin) === null || _a === void 0 ? void 0 : _a.map((plugin) => {
                this.registerPlugin(plugin);
            });
            this.factory.forEach((factory) => {
                this.loading(factory.id, factory.use);
            });
        }
    }
    // 注册插件
    registerPlugin(plugin) {
        const { name } = plugin;
        if (plugin.generatedHook) {
            generatedHook.tapPromise(name, plugin.generatedHook);
        }
    }
    // 开辟存储区域
    createStorage(factoryId, source) {
        this.storage[factoryId] = source;
    }
    // 根据 factoryID 获取 factory option
    getFactory(factoryId) {
        const [factory] = this.factory.filter((it) => it.id === factoryId);
        return factory;
    }
    // 加载器运行
    loading(factoryId, loads) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const source = yield Promise.all(loads.map((it) => __awaiter(this, void 0, void 0, function* () {
                    return yield it(this.ctx);
                })));
                this.createStorage(factoryId, source);
                // NOTE: 加载数据
                this.log.info('load data...');
                this.run(factoryId);
            }
            catch (error) { }
        });
    }
    run(factoryId) {
        const factory = this.getFactory(factoryId);
        const { machine } = factory;
        const source = this.storage[factoryId];
        this.log.debug('[meta data]', source);
        const files = machine(source, this.ctx);
        this.log.info('generate code...');
        // NOTE: 要触发对应的钩子
        generatedHook.promise(files).then((res) => {
            this.writeFile(res, factory.output || this.option.output);
        });
    }
    writeFile(files, rootPath) {
        files.forEach((file) => {
            if (file.pathname && file.code) {
                if (path.isAbsolute(file.pathname)) {
                    this.log.debug(file.pathname, file.code);
                    // NOTE: 判断路径有效
                    fs.outputFileSync(file.pathname, file.code);
                }
                else {
                    if (rootPath) {
                        const filePath = path.resolve(rootPath, file.pathname);
                        this.log.debug(filePath, file.code);
                        // NOTE: 判断路径有效
                        fs.outputFileSync(filePath, file.code);
                    }
                    else {
                        this.log.warn('miss root path');
                    }
                }
            }
        });
        this.log.success('success');
    }
}
